@@ -9,9 +9,11 @@ def main():
         'test/sanity/code-smell/%s' % os.path.basename(__file__),
         # facts is grandfathered in but will break namespacing
         # the only way to fix it is to deprecate and eventually remove it
-        # six will break namespacing but because it is bundled we should not be overriding it
+        # six and distro will break namespacing but because it is bundled we should not be
+        # overriding it
         'lib/ansible/module_utils/facts/__init__.py',
         'lib/ansible/module_utils/six/__init__.py',
+        'lib/ansible/module_utils/distro/__init__.py',
     ])
 
     for path in sys.argv[1:] or sys.stdin.read().splitlines():
